@@ -10,25 +10,34 @@
 #
 
 import json
-import sys
+import os
+from typing import Dict
+from typing import List
 
 import click
 from packaging.requirements import Requirement
+from tinynetrc import Netrc
 
+from _packagedcode.models import DependentPackage
+from _packagedcode.pypi import PipRequirementsFileHandler
+from _packagedcode.pypi import can_process_dependent_package
 from python_inspector import dependencies
+from python_inspector import utils
 from python_inspector import utils_pypi
 from python_inspector.cli_utils import FileOptionType
+from python_inspector.resolution import get_environment_marker_from_environment
 from python_inspector.resolution import get_resolved_dependencies
 
 TRACE = False
 
-__version__ = "0.5.0"
+__version__ = "0.6.3"
 
 DEFAULT_PYTHON_VERSION = "38"
 PYPI_SIMPLE_URL = "https://pypi.org/simple"
 
 
 @click.command()
+@click.pass_context
 @click.option(
     "-r",
     "--requirement",
@@ -40,6 +49,15 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     help="Path to pip requirements file listing thirdparty packages. "
     "This option can be used multiple times.",
 )
+@click.option(
+    "-n",
+    "--netrc",
+    "netrc_file",
+    type=click.Path(exists=True, readable=True, path_type=str, dir_okay=False),
+    metavar="NETRC-FILE",
+    required=False,
+    help="Netrc file to use for authentication. ",
+)
 @click.option(
     "--spec",
     "--specifier",
@@ -85,11 +103,27 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     "--json",
     "json_output",
     type=FileOptionType(mode="w", encoding="utf-8", lazy=True),
-    required=True,
+    required=False,
     metavar="FILE",
     help="Write output as pretty-printed JSON to FILE. "
     "Use the special '-' file name to print results on screen/stdout.",
 )
+@click.option(
+    "--json-pdt",
+    "pdt_output",
+    type=FileOptionType(mode="w", encoding="utf-8", lazy=True),
+    required=False,
+    metavar="FILE",
+    help="Write output as pretty-printed JSON to FILE as a tree in the style of pipdeptree. "
+    "Use the special '-' file name to print results on screen/stdout.",
+)
+@click.option(
+    "--max-rounds",
+    "max_rounds",
+    type=int,
+    default=200000,
+    help="Increase the max rounds whenever the resolution is too deep",
+)
 @click.option(
     "--use-cached-index",
     is_flag=True,
@@ -103,22 +137,26 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     "--index-url are ignored when this option is active.",
 )
 @click.option(
-    "--debug",
+    "--verbose",
     is_flag=True,
     hidden=True,
     help="Enable debug output.",
 )
 @click.help_option("-h", "--help")
 def resolve_dependencies(
+    ctx,
     requirement_files,
+    netrc_file,
     specifiers,
     python_version,
     operating_system,
     index_urls,
     json_output,
+    pdt_output,
+    max_rounds,
     use_cached_index=False,
     use_pypi_json_api=False,
-    debug=TRACE,
+    verbose=TRACE,
 ):
     """
     Resolve the dependencies of the packages listed in REQUIREMENT-FILE(s) file
@@ -136,16 +174,47 @@ def resolve_dependencies(
     For example, display the results of resolving the dependencies for flask==2.1.2
     on screen::
 
-        dad --spec "flask==2.1.2" --json -
+        python-inspector --spec "flask==2.1.2" --json -
     """
+    if not (json_output or pdt_output):
+        click.secho("No output file specified. Use --json or --json-pdt.", err=True)
+        ctx.exit(1)
 
-    click.secho(f"Resolving dependencies...")
+    if json_output and pdt_output:
+        click.secho("Only one of --json or --json-pdt can be used.", err=True)
+        ctx.exit(1)
+
+    if verbose:
+        click.secho(f"Resolving dependencies...")
+
+    if netrc_file:
+        if not os.path.exists(netrc_file):
+            raise Exception(f"Missing netrc file {netrc_file}")
+
+    if not netrc_file:
+        netrc_file = os.path.join(os.path.expanduser("~"), ".netrc")
+        if not os.path.exists(netrc_file):
+            netrc_file = os.path.join(os.path.expanduser("~"), "_netrc")
+            if not os.path.exists(netrc_file):
+                netrc_file = None
+
+    if netrc_file:
+        if verbose:
+            click.secho(f"Using netrc file {netrc_file}")
+        netrc = Netrc(file=netrc_file)
+    else:
+        netrc = None
 
     # TODO: deduplicate me
     direct_dependencies = []
 
+    if PYPI_SIMPLE_URL not in index_urls:
+        index_urls = tuple([PYPI_SIMPLE_URL]) + tuple(index_urls)
+
     for req_file in requirement_files:
         deps = dependencies.get_dependencies_from_requirements(requirements_file=req_file)
+        for extra_data in dependencies.get_extra_data_from_requirements(requirements_file=req_file):
+            index_urls = (*index_urls, *tuple(extra_data.get("extra_index_urls") or []))
         direct_dependencies.extend(deps)
 
     for specifier in specifiers:
@@ -154,9 +223,9 @@ def resolve_dependencies(
 
     if not direct_dependencies:
         click.secho("Error: no requirements requested.")
-        sys.exit(1)
+        ctx.exit(1)
 
-    if debug:
+    if verbose:
         click.secho("direct_dependencies:")
         for dep in direct_dependencies:
             click.secho(f" {dep}")
@@ -166,7 +235,7 @@ def resolve_dependencies(
         python_version=python_version, operating_system=operating_system
     )
 
-    if debug:
+    if verbose:
         click.secho(f"environment: {environment}")
 
     repos = []
@@ -179,13 +248,20 @@ def resolve_dependencies(
                 existing.use_cached_index = use_cached_index
                 repos.append(existing)
             else:
+                credentials = None
+                if netrc:
+                    login, password = utils.get_netrc_auth(index_url, netrc)
+                    credentials = (
+                        dict(login=login, password=password) if login and password else None
+                    )
                 repo = utils_pypi.PypiSimpleRepository(
                     index_url=index_url,
                     use_cached_index=use_cached_index,
+                    credentials=credentials,
                 )
                 repos.append(repo)
 
-    if debug:
+    if verbose:
         click.secho("repos:")
         for repo in repos:
             click.secho(f" {repo}")
@@ -196,6 +272,9 @@ def resolve_dependencies(
         environment=environment,
         repos=repos,
         as_tree=False,
+        max_rounds=max_rounds,
+        verbose=verbose,
+        pdt_output=pdt_output,
     )
 
     cli_options = [f"--requirement {rf}" for rf in requirement_files]
@@ -212,7 +291,7 @@ def resolve_dependencies(
     )
 
     headers = dict(
-        tool_name="dad",
+        tool_name="python-inspector",
         tool_homepageurl="https://github.com/nexB/python-inspector",
         tool_version=__version__,
         options=cli_options,
@@ -221,18 +300,36 @@ def resolve_dependencies(
         errors=[],
     )
 
-    write_output(
-        headers=headers,
-        requirements=requirements,
-        resolved_dependencies=resolved_dependencies,
-        json_output=json_output,
-    )
+    if json_output:
+        write_output(
+            headers=headers,
+            requirements=requirements,
+            resolved_dependencies=resolved_dependencies,
+            json_output=json_output,
+        )
 
-    if debug:
+    else:
+        write_output(
+            headers=headers,
+            requirements=requirements,
+            resolved_dependencies=resolved_dependencies,
+            json_output=pdt_output,
+            pdt_output=True,
+        )
+
+    if verbose:
         click.secho("done!")
 
 
-def resolve(direct_dependencies, environment, repos=tuple(), as_tree=False):
+def resolve(
+    direct_dependencies,
+    environment,
+    repos=tuple(),
+    as_tree=False,
+    max_rounds=200000,
+    verbose=False,
+    pdt_output=False,
+):
     """
     Resolve dependencies given a ``direct_dependencies`` list of
     DependentPackage and return a tuple of (initial_requirements,
@@ -241,14 +338,22 @@ def resolve(direct_dependencies, environment, repos=tuple(), as_tree=False):
     If empty, use instead the PyPI.org JSON API exclusively.
     """
 
-    requirements = [
-        Requirement(requirement_string=d.extracted_requirement) for d in direct_dependencies
-    ]
+    environment_marker = get_environment_marker_from_environment(environment)
+
+    requirements = list(
+        get_requirements_from_direct_dependencies(
+            direct_dependencies=direct_dependencies, environment_marker=environment_marker
+        )
+    )
+
     resolved_dependencies = get_resolved_dependencies(
         requirements=requirements,
         environment=environment,
         repos=repos,
         as_tree=as_tree,
+        max_rounds=max_rounds,
+        verbose=verbose,
+        pdt_output=pdt_output,
     )
 
     initial_requirements = [d.to_dict() for d in direct_dependencies]
@@ -256,16 +361,36 @@ def resolve(direct_dependencies, environment, repos=tuple(), as_tree=False):
     return initial_requirements, resolved_dependencies
 
 
-def write_output(headers, requirements, resolved_dependencies, json_output):
+def get_requirements_from_direct_dependencies(
+    direct_dependencies: List[DependentPackage], environment_marker: Dict
+) -> List[Requirement]:
+    for dependency in direct_dependencies:
+        # FIXME We are skipping editable requirements
+        # and other pip options for now
+        # https://github.com/nexB/python-inspector/issues/41
+        if not can_process_dependent_package(dependency):
+            continue
+        req = Requirement(requirement_string=dependency.extracted_requirement)
+        if req.marker is None:
+            yield req
+        else:
+            if req.marker.evaluate(environment_marker):
+                yield req
+
+
+def write_output(headers, requirements, resolved_dependencies, json_output, pdt_output=False):
     """
     Write headers, requirements and resolved_dependencies as JSON to ``json_output``.
     Return the output data.
     """
-    output = dict(
-        headers=headers,
-        requirements=requirements,
-        resolved_dependencies=resolved_dependencies,
-    )
+    if not pdt_output:
+        output = dict(
+            headers=headers,
+            requirements=requirements,
+            resolved_dependencies=resolved_dependencies,
+        )
+    else:
+        output = resolved_dependencies
 
     json.dump(output, json_output, indent=2)
     return output
